// This file is common to the XML Moo Database browser and Moo Canada's IDE.


////////// Rendering Functions //////////

var RenderQueue = [];
var RenderTask = null;

function renderQueueAdd(node) {
  // Place the node on the queue of nodes to be rendered.
  // Don't add duplicates to the queue.
  if (node == null) {
    return;
  }
  for (var x = 0; x < RenderQueue.length; x++) {
    if (node === RenderQueue[x]) {
      return;
    }
  }
  // Add node to the queue.
  RenderQueue.push(node);
  renderQueueSoon();
}

function renderQueueSoon() {
  // Schedule a rerender in 10ms.
  if (RenderTask) {
    clearTimeout(RenderTask);
  }
  RenderTask = setTimeout("renderQueueNow()", 10);
}

function renderQueueNow() {
  // Render everything which has been placed in the queue.
  RenderTask = null;
  for (var x = 0; x < RenderQueue.length; x++) {
    renderNode(RenderQueue[x]);
  }
  // Empty the queue.
  RenderQueue = [];
}

function renderNode(node) {
  // Create the HTML for this node.
  
  // The first element is the line for this object, any other elements are children.
  var line = node.firstChild;

  var control = document.createElement("IMG");
  if (line != node.lastChild) {
    if (node.lastChild.style.display == "none") {
      control.src = "tree-closed.gif";
    } else {
      control.src = "tree-opened.gif";
    }
  } else {
    control.src = "tree-node.gif";
  }
  control.onclick = objClickFold;
  control.className = "control";

  var label = document.createElement("SPAN");
  label.appendChild(document.createTextNode(line.title));
  label.onclick = objClickName;

  // Is it a selected node?
  for (var x in SelectedNodes) {
    if (SelectedNodes[x] === node) {
      label.className = 'selected';
    }
  }

  // Empty the line.
  while(line.firstChild) {
    line.removeChild(line.firstChild);
  }
  // Add the new contents.
  line.appendChild(control);
  line.appendChild(label);
}

////////// Tree functions //////////

function clear() {
  // Delete the 'Loading...' message in the first two tabs.
  // Maybe this should be genalised in the future.
  while (Tab.List['inheritance'].contentNode.firstChild) {
    Tab.List['inheritance'].contentNode.removeChild(
        Tab.List['inheritance'].contentNode.firstChild);
  }
  while (Tab.List['location'].contentNode.firstChild) {
    Tab.List['location'].contentNode.removeChild(
        Tab.List['location'].contentNode.firstChild);
  }
}

// Look up the node and pass it to any interested tabs.
function treeUpdateId(id) {
  var node = parent.parent.getXmlNodeById(id);
  for (var x in Tab.List) {
    if (Tab.List[x].updateObject) {
      Tab.List[x].updateObject(node);
    }
  }
}

if (Tab.List['inheritance']) {
  Tab.List['inheritance'].updateObject = function(xmlNode) {
    var id = xmlNode.getAttribute('ID');
    var number = id.replace('obj', '#');
    var htmlNode = treeGetOrCreateObject(number, this.name);
    var htmlChildNode, childNumber;

    // TODO: remove children which don't belong.
    var name = "";
    var childXmlNodeLength = xmlNode.childNodes.length;
    var childXmlNode
    for (var x = 0; x < childXmlNodeLength; x++) {
      childXmlNode = xmlNode.childNodes[x];
      if (childXmlNode.nodeName == "NAME" && childXmlNode.firstChild) {
        name += childXmlNode.firstChild.nodeValue;
      } else if (childXmlNode.nodeName == "OBJECT") {
        childNumber = childXmlNode.getAttribute('ID').replace('obj', '#');
        htmlChildNode = treeGetOrCreateObject(childNumber, this.name);
        treeAttachChildToParent(htmlChildNode, htmlNode);
      }
    }
    name += " ("+number+")";
    htmlNode.firstChild.title = name;
  }
}

if (Tab.List['location']) {
  Tab.List['location'].updateObject = function(xmlNode) {
    var id = xmlNode.getAttribute('ID');
    var number = id.replace('obj', '#');
    var htmlNode = treeGetOrCreateObject(number, this.name);

    var name = "";
    var childXmlNodeLength = xmlNode.childNodes.length;
    var childXmlNode
    for (var x = 0; x < childXmlNodeLength; x++) {
      childXmlNode = xmlNode.childNodes[x];
      if (childXmlNode.nodeName == "NAME" && childXmlNode.firstChild) {
        name += childXmlNode.firstChild.nodeValue;
      }
    }
    name += " ("+number+")";
    htmlNode.firstChild.title = name;

    var childNumber = xmlNode.getAttribute('LOCATION').replace('obj', '#');
    if (childNumber == "#-1") {
      treeAttachChildToParent(htmlNode, null);
    } else {
      var htmlParentNode = treeGetOrCreateObject(childNumber, this.name);
      treeAttachChildToParent(htmlNode, htmlParentNode);
    }
  }
}

function treeAttachChildToParent(child, parent) {
  if (parent == null) {
    parent = child.parentNode;
    while(parent && parent.className != "tabcontent") {
      parent = parent.parentNode;
    }
    if (!parent) {
      // Can't happen.
      alert("treeAttachChildToParent: ran off the top of the page.");
      return;
    }
  }
  if (parent === child.parentNode && parent.className != "tabcontent") {
    // No change needed
    return;
  }
  // Make this child as visible as its siblings.
  var visible = true;
  if (parent.lastChild && parent.lastChild.style.display == "none") {
    visible = false;
  }
  treeSetFold(child, visible);
  // Insert the node in the right order.
  for (var x = 0; x < parent.childNodes.length; x++) {
    if (parent.childNodes[x].id && parent.childNodes[x].id > child.id) {
      parent.insertBefore(child, parent.childNodes[x]);
      return;
    }
  }
  parent.appendChild(child);
}

function treeGetOrCreateObject(number, tab) {
  // Return the node for the requested object, or create it if it doesn't exist.
  var id = tab + "_obj" + number.substring(1); // Strip off the leading '#'
  var node = document.getElementById(id);
  if (node == null) {
    node = document.createElement("DIV");
    node.id = id;
    node.className = "object";
    // Create empty span where control and text will go.
    var span = document.createElement("SPAN");
    span.className = "object";
    span.title = number;
    node.appendChild(span);
    document.getElementById("content_"+tab).appendChild(node);
    renderQueueAdd(node);
  }
  return node;
}


function treeGetObject(number, tab) {
  // Return the node for the requested object.
  var id = tab + "_obj" + number.substring(1); // Strip off the leading '#'
  return document.getElementById(id);
}

function objClickFold() {
  var node = this.parentNode.parentNode;
  treeSetFold(node, node.lastChild.style.display == "none");
  renderQueueAdd(node);
  renderQueueSoon();
}

function treeUnfoldToNode(node) {
  if (!node) {
    return;
  }
  while ((node = node.parentNode).className == "object") {
    treeSetFold(node, true)
  }
}

function treeFoldAll(open) {
  var divs = document.getElementsByTagName("DIV");
  for (var x = 0; x < divs.length; x++) {
    if (divs[x].className == "object") {
      treeSetFold(divs[x], open);
    }
  }
}

function treeSetFold(node, open) {
  var display = open ? "block" : "none";
  for (var x = 1; x < node.childNodes.length; x++) {
    node.childNodes[x].style.display = display;
  }
}

function objClickName() {
  var node = this.parentNode.parentNode;

  // Call over to the list frame and switch to the new object.
  var xmlId = node.id;
  xmlId = xmlId.substring(xmlId.indexOf("_") + 1);
  parent.parent.ListFrame.changeObject(xmlId);
  treeSetFold(node, true)

  treeSelect(xmlId);
  renderQueueSoon();
}

var SelectedNodes = new Object();
for (var x in Tab.List) {
  SelectedNodes[x] = null;
  if (Tab.List[x].oldOnClick) {
    Tab.List[x].oldSelectOnClick = Tab.List[x].oldOnClick
  }
  // Add a hook to each tab so that if it is a tree it will
  // unfold to the currently selected node.
  Tab.List[x].oldOnClick = function() {
    treeUnfoldToNode(SelectedNodes[this.name]);
    // Causes Firefox to scroll whole page up resulting in lost tabs.
    //SelectedNodes[this.name].scrollIntoView();
    if (this.oldSelectOnClick) {
      this.oldSelectOnClick();
    }
  }
}

function treeSelect(objid) {
  for (var x in Tab.List) {
    renderQueueAdd(SelectedNodes[x]);
    node = document.getElementById(x + "_" + objid);
    SelectedNodes[x] = node;
    renderQueueAdd(node);
  }
  renderQueueSoon();
}

function changeObject(number) {
  // Call over to the list frame and switch to the new object.
  parent.parent.ListFrame.changeObject(number);
  treeSelect(number);
  renderQueueSoon();
  // Unfold the currently selected tree to this node.
  // This could happen if the object is changed from elsewhere.
  var selectedTabName = Tab.Selected();
  if (selectedTabName) {
    var node = document.getElementById(selectedTabName + "_" + number);
    treeUnfoldToNode(node);
    node.scrollIntoView();
  }
}

